import java.net.*;
import com.interfaceware.chameleon.*;

public class ConsoleServer implements Log
{
   private static final String MESSAGE_DEFINITION_FILE = "project.vmd";

   private DynamicEngine m_engine;
   
	static public void main(String[] Args)
   {
		ConsoleServer Server = new ConsoleServer();
      Server.startServer();
   }

   public ConsoleServer() {}

   public void startServer()
   {
      try
      {
         m_engine = new DynamicEngine();
         m_engine.addLog(this);
         m_engine.load(MESSAGE_DEFINITION_FILE);
         // comment the following line for strict segment grammar validation
         m_engine.setRejectBadSegmentGrammar(false);

         // instantiate a table object
         Table MessageTable = new Table();
         println("This example shows how one can avoid using stub code in Chameleon");
         println("and instead use a dynamic interface to example data within the engine.");
         String FlatWire;
         FlatWire = "MSH|^~\\&|REGADT|MCM|RSP1P8|MCM|199601051530|SEC|ADT^A44|00000007|P|2.3\r";
         FlatWire = FlatWire + "EVN|A44|199601051530\r";
         FlatWire = FlatWire + "PID|||MR2^^^XYZ||JONES^WILLIAM^A^JR||19501010|M|||123 EAST STREET^^NY^NY^10021||(212)111-3333|||S||ACCT1\r";
         FlatWire = FlatWire + "MRG|MR1^^^XYZ||ACCT1\r";
         FlatWire = FlatWire + "MRG|MR1^^^XYZ||ACCT1\r";
         FlatWire = FlatWire + "MRG|MR1^^^XYZ||ACCT1\r";
         
         int MessageIndex = m_engine.parseMessageDynamically(FlatWire, MessageTable);
         println("Got a " + m_engine.messageName(MessageIndex));

         examineTable("", MessageTable);

	 println("Press Enter to Exit...");
         System.in.read();
      }
      catch (ChameleonException err)
      {
         println(err.toString());
      }
      catch (java.io.IOException err)
      {
         println(err.toString());
      }
   }

   public void examineTable(String Indent, Table ATable) throws ChameleonException
   {
      println(Indent + "Table Name:" + ATable.tableName());
      println(Indent + " count of row = " + ATable.countOfRow());
      for (int RowIndex = 0;
           RowIndex < ATable.countOfRow();
           RowIndex++)
      {
         for (int SubTableIndex = 0;
              SubTableIndex < ATable.countOfSubTable();
              SubTableIndex++)
         {
            examineTable(Indent + "   ", ATable.subTable(RowIndex, SubTableIndex));
         }
         for (int ColumnIndex = 0;
              ColumnIndex < ATable.countOfColumn();
              ColumnIndex++)
         {
            String ColumnOutput;
            ColumnOutput = Indent + ATable.columnName(ColumnIndex) + ": ";
            switch (ATable.columnType(ColumnIndex))
            {
            case DataType.StringType:
                ColumnOutput += ATable.getString(ColumnIndex, RowIndex);
                break;
            case DataType.DoubleType:
                ColumnOutput += ATable.getDouble(ColumnIndex, RowIndex);
                break;
            case DataType.IntegerType:
                ColumnOutput += ATable.getInteger(ColumnIndex, RowIndex);
                break;
            case DataType.DateType:
                ColumnOutput += ATable.getDate(ColumnIndex, RowIndex);
                break;
            }
            println(ColumnOutput);
         }
      }
   }

   // Log interface

   public void println(String output_text)
   {
      System.out.println(output_text);		
   }
}
